function [roots,orders] = HOM4PS2(n,f,mthd)
%      Homotopy method for solving polynomial systems.
%
%  Syntax:   >> [rootCell,nroot] = HOM4PS2(n,P,mthd)
%  Input :
%           n -- the number of equations.
%           P -- polynomial system (string form).
%        mthd -- (optional)
%                = 1 : use the polyhedral homotopy method. (default)
%                = 2 : use the classical homotopy method.
%
%  output :
%    rootCell -- Matlab Cell containing information of solutions.
%             rootCell{:,1} : solutions.
%             rootCell{:,2} : the residue of solutions.
%             rootCell{:,3} : the condition number of solutions.
%
%        e.g. rootCell{1,1} is the first solution.
%             rootCell{1,2} is the residue of the first solution.
%             rootCell{1,3} is the condition number of the first solution.
%
%       nroot -- the number of solutions.
%
%  Example:
% >> P = '{3+x^3*y^2+x; 4*y*z^5+8*x^2*y^4*z^4-1; x+y+z-1;}'
% >> [rootCell,nroot] = HOM4PS2(3,P);
%

P='{';
for i=1:n
	if   i==n
	    P = [P,char(f(i)),'}'];
	else
	    P = [P,char(f(i)),';'];
	end 
end %end for
disp(['The length of polynomial string :        ', int2str(length(P))]);

if (nargin < 2)
   error('Two inputs required.')
end

if (nargin < 3)
    mthd = 1;
end

if ~(mthd == 1 | mthd == 2)
    error('(The third input (mthd) should be 1 or 2.')
end
b = findsym(f);
%----------------------------------------------

   fid = fopen('input.sym','w');
   fprintf(fid,'%s',P);
   fclose(fid);

   filepath =  which('hom4ps2.bat')
   
   eval(['!',filepath,'   input.sym'])
   

%javaaddpath('.\ParseData.jar');

strvars = findsym(f);
strvars = strrep(strvars,',',' ');

%mlscript = javaMethod('parseData','ParseData','data.roots','','roots','orders')
%mlscript
%eval(['syms  ', strvars])
%eval(char(mlscript));
 [roots,orders] = analysedata('data.roots');
 delete data.roots
end
