function p = consRegByMinor(L)
%  CONSREGBYMINOR        Use 'Minor' method to construct new polynomial system
%
%The instruction
%  
%  p = CONSREGBYMINOR(L, option)
%
%  Input : 
%       L -- initial polynomial system
%       option -- 
% Output :
%       p -- new polynomial system
%
% Yijun. Zhu, 28 March 2013
%
varstr = findsym(L);
vars = sym(['[',varstr,']']);
varNum = length(vars); % number of variables
LNum = length(L); % number of polynomials
if (varNum == LNum)
    p = {L};
    return;
end
coefHoriVec = myrandom(-1,1,1,varNum); %random vector
horiVec = vars - coefHoriVec; 
horiVec = 1/2*horiVec.*horiVec;
g = sum(horiVec); % generate g polynomial
newL = [L,g];
jacMatTrans = jacobian(newL).';
index = 1: varNum;
combs = nchoosek(index,LNum+1); 
combsCell = mat2cell(combs,ones(1,size(combs,1)),LNum+1);
newPolys = sym(zeros(1,length(combsCell)));
% Get every sub matrix, and compute the value of its determinant
for i = 1 : length(combsCell)
    subMat = jacMatTrans(combsCell{i},:); % get sub matrix
    newPolys(i) = det(subMat);
end
newL = linearComb(newPolys,varNum - LNum); %  combime all new polynomials into (n-m) polynomials
newL = [L,newL];
p = {vpa(newL)};
end