function f = construct_regular(L)
%  CONSTRUCT_REGULAR       Given an underdetermined system, construct several regular systems
%
%The instruction
%  
%  f = CONSTRUCT_REGULAR(L)
%
% Yijun. Zhu, 28 March 2013
%

 m = length(L);
 varsstr = findsym(L);
 varsstr = ['[',varsstr,']'];
 vars = sym(varsstr);
 n = length(vars);
 if m == n  ;
     f = {L};
     return ;
 end
 J_f = jacobian(L,vars);
    
 % Choose a random vector whose elments are in the range (-3..3).
 anchor = randsample((-3000:3000)/1000,n);
 
 system_list={};
 % Construct several regular systems:
 for i = 0 : n-m 
     temp_list=L;
     J_sub=J_f(1:m,1:m+i-1);
     % Construct i minors randomly from the submatrix J_sub 
     if i~=0 
         column_list=mychoose(m+i-1,m);
         num_column_list=length(column_list);
         num_list=myrandcomb(num_column_list,i);
         for j = 1 : i          
             J_sub_sub=J_sub(1:m,column_list{num_list(j)});
             aux_f=expand(det(J_sub_sub));
             temp_list=[temp_list,aux_f];
         end;
     end;
            
     % Construct n-m-i linear systems.
     if i == n-m 
         system_list=[system_list(1:length(system_list)),{temp_list}];
     else
         for j = 1 : n-m-i
             temp_list=[temp_list,vars(n+1-j)-anchor(n+1-j)];
         end;
         system_list=[system_list(1:length(system_list)),{temp_list}];
     end;
 end;
 f=system_list;
 return;
end  


function mat = mychoose(n,m)
m = nchoosek(1:n,m);
[r,c]= size(m);
mat = mat2cell(m,ones(r,1),c);
end

function f = myrandcomb(m,n)
% Return n numbers selected from 1 to m  randomly
rand=randperm(m);%Get a random sequence from 1 to m
index=rand(1:n);
index=sort(index);
temp=1:m;
f=temp(index);

end
              
       
    
   
     
    
