function [X] = AFPC_BB_moment_ineq(n,A_eq,b_eq,A_ineq,num_b_ineq,maxinneriter,tol,eta,...
                            continuation,schur_type)
  
   
   if ~strcmp(schur_type,'full') && ~strcmp(schur_type,'partial')
      error(' schur_type must be ''full'' or ''partial'' '); 
   end
   
   
   A = [A_eq;A_ineq];
   b = [b_eq;zeros(sum(num_b_ineq),1)];
   
   
   m = length(num_b_ineq); % number of inequalities
   num_b = length(b_eq); 
   
   
   
   p = length(b);
   %sr = p/n^2;
   %fr = r*(2*n-r+1)/2/p;
   fprintf('p:%d\n',p);
   %fprintf('sr:%f\n',sr);
   %fprintf('fr:%f\n',fr);
   
   %if fr< 0.4 || fr >= 0.4
   %    fprintf('This is a "hard" problem! \n');
   %else
   %    fprintf('This is an "easy" problem! \n');
   %end   
   
   
   time1 = cputime;
    
   maxiter = 1e+5; % maximum iteration number for the outer loop
   maxr = floor(n/10);  
   fast_eig_ratio = 0.001; % ratio for computing the rank for next iteration
   normb = norm(b);
   
%% estimate Lipschitz constant = largest eigenvalue of Map_A(Adj_A). 
%%
    
      options.tol   = 1e-6; 
      options.issym = true; 
      options.disp  = 0; 
      options.v0    = randn(p,1);
      Lipschitz_const = eigs(@(y)Map_A(A,Adj_A(A,y,n)),p,1,'LM',options); 
      tau = 2/full(Lipschitz_const);  
      taumax = 5*tau;
      taumin = 1e-3*tau;
%%
%% initial mu and target mu

   temp = Adj_A(A,b,n);  
   
   if n < 500
          nrm2Adjb = norm(temp);
   else
          T = laneig(temp,3); 
          nrm2Adjb = T(1);
   end
   mu_final = 1e-4*nrm2Adjb;
   
   if continuation
       mu = eta*nrm2Adjb;
   else 
       mu = mu_final;
   end
   if mu < mu_final, mu = mu_final; end
%%  
%% Initialization
         
   X = zeros(n,n);
   Xold = X;
   t = 1;
   extra_rank = 0; 
   violate_num = 0;
   norm_old = inf;
   numin = 0;
   
%% main loop
%%
   fprintf('AFPC-BB:\n');
   fprintf('Iteration begins:\n');
   fprintf('*******************************************************\n');
   
   for iter = 1:maxiter
          
          if mu == mu_final
                 maxinneriter = 5000;
          end

          if (mu > mu_final)
             eigtol = 0.1*min([1e-3,1e-2*mu]);  
          else
             eigtol = 0.1*min([1e-5,1e-1*tol,1e-2*mu]); 
          end

          if iter == 1
              G = tau*temp;
          else
              % compute b first
              flag =0;
              k = num_b;
              for i = 1:m
                  AA = A_ineq(flag+1:flag+num_b_ineq(i),:);
                  dim = (-1+sqrt(1+8*num_b_ineq(i)))/2;
                  flag = flag+num_b_ineq(i);
                  s = Map_A(AA,X);
                  S = vec2mat(s,dim);
                  
                  [U, T] = schur(S);
                  t = diag(T);      
                  ind = find(t>0);         
                  TT = diag(t(ind)); 
                  UU = U(:,ind);                       
                  S = UU*TT*UU'; 
                  
                  b(k+1:k+num_b_ineq(i)) = mat2vec(S);                 
                  k = k+num_b_ineq(i);
              end
                  
                      
              t     = (1+sqrt(1+4*told^2))/2;
              beta  = (told-1)/t;
              Y    = (beta+1)*X - beta*Xold; 
              AY_b  = Map_A(A,Y)-b;
              Grad  = Adj_A(A,AY_b,n);   
              G     = Y - tt*Grad;  
          end
          Xold = X;
          
          if iter ==1
              if n < 1000 
                     [U, T] = schur(G);  
              else
                     [U, T] = laneig(G,maxr,'LM',eigtol);
              end
          else
              if  strcmp(schur_type,'full')
                  [U, T] = schur(G);
              else
                  Te_old = Te(Te>0); mr = max(Te_old);
                  kk = length(find(Te_old > mr*fast_eig_ratio)); 
                  kk = max(1,min(kk+extra_rank,n));  %disp(kk);
                  G = (G+G')/2; 
                  [U, T] = laneig(G,kk,'LM',eigtol); 
              end
              
          end
                    
          nu = tau*mu; 
          sigma = diag(T);      
          ind = find(sigma > nu); r = length(ind);         
          Te = diag(sigma(ind)-nu); 
          Ue = U(:,ind);                       
          X = Ue*Te*Ue'; 
          
          rX = X-Xold;
          norm_new = norm(rX,'fro');
          normXold = max(norm(Xold,'fro'),1);
          residualX = norm_new/normXold;
          
          AX_b = Map_A(A,X)-b;
          normAX1_b = norm(AX_b);   
          residualA = normAX1_b/normb;  
          
          numin = numin +1;
          fprintf('***************************\n');
          fprintf('Iteration:%d\n',numin);
          fprintf('Rank:%d\n',r);
          fprintf('Residual_X:%6.8e\n',residualX);
          fprintf('The relative error of AX-b is %6.8e\n',residualA);
          %fprintf('The nuclear norm is %6.8f\n',norm(diag(Te),1));
          %no need to compute in large scale examples
          told = t; 
          
          if iter ~= 1 
              if norm_new > norm_old
                  violate_num = violate_num + 1;
                  if violate_num >= 10
                      extra_rank = 1;
                      violate_num = 0;
                  else
                      extra_rank = 0;
                  end
              end
          end
         
          norm_old = norm_new;
        
          
          if residualA > 1e5
              disp('Divergence!');
              break
          end
          
          %% check stopping criterion
          if residualA < tol || numin >= maxinneriter
              numin = 0;
              fprintf('*******************************************************\n');
              if mu == mu_final
                  eTime = cputime - time1; 
                  %fprintf('Rank:%d\n',r);
                  %fprintf('Residual_X:%6.8e\n',residualX);
                  %fprintf('The relative error of AX-b is %6.8e\n',residualA);
                  fprintf('The number of iteration is %d\n',iter);
                  fprintf('The computing time is %f\n\n',eTime);
                  return
              end
              %% update mu
              mu = max(mu*eta,mu_final);
          end
          
          % update the step size  by Barzilai-Borwein technique
              ArX = Map_A(A,rX);
              rg = Adj_A(A,ArX,n);
              tr_rXrg = sum(sum(rX.*rg));
              tr_rgrg = sum(sum(rg.*rg));
              tt = tr_rXrg/tr_rgrg; 
              tt = max(taumin,min(tt,taumax));               

%           % check
%              MM1 = A_eq'*A_eq;
%              MM2 = A_ineq'*A_ineq;
%              mm = size(MM1,1);
%              MM = eye(mm)-tt*MM1-tt*MM2;
%              che = norm(full(MM))+tt*norm(full(MM2));
%              disp(che);
%              
          
   end
end
   
%%
%%
%-------------------------------------------------------------------------%
%                            Subfunctions
%-------------------------------------------------------------------------%

   function y = Map_A(A,X)
%   y = A(X)    
       [m1,m2] = size(X);
       x = reshape(X,m1*m2,1);
       y = A*x;
   end
       
       
%-------------------------------------------------------------------------%

   function X = Adj_A(A,y,n)
%    X=A^*(y)
      x = A'*y;
      if length(x)==n^2
          X = reshape(x,n,n);
      end 
      X = X+X';
      for i = 1:n
          X(i,i) = X(i,i)/2;
      end
   end
      
%-------------------------------------------------------------------------%

  function X = vec2mat(y,n)
      X = zeros(n,n);
      k = 1;
      for j=1:n
          for i=1:j
              X(i,j) = y(k);
              k = k+1;
          end
      end
      X = X+X';
      for i = 1:n
          X(i,i) = X(i,i)/2;
      end
  end

  function s = mat2vec(S)
      n = size(S,1);
      m = n*(n+1)/2;
      s = zeros(m,1);
      k = 1;
      for j=1:n
          for i=1:j
              s(k) = S(i,j);
              k = k+1;
          end
      end
  end
%%
%%