function [out,count] = fixVerification(symK, vars, sol)
% FIXVERIFICATION         Use 'fix' method  to construct a new polynomials system and then verify it
%
%The instruction
%  
%   [out, count] = FIXVERIFICATION(symK,vars,sol)
%
%  Input : 
%       symK -- sym type polynomials 
%       vars -- variables
%       sol -- sample points
% Output :
%       out -- results
%       count -- number of results
%
% Yijun. Zhu, 28 March 2013
%
out = {};
pnum = length(symK); % number of polynomials
vnum = size(sol,1); % number of variables
solnum = size(sol,2); % number of sample points 

repnum = vnum - pnum; % number of variables to replace
repvars = vars(end-repnum+1 : end); % variables to replace
repvalues = sol(end-repnum+1 : end,:); % values to replace with

%Generate LL  
LL = {};
for i = 1 : solnum
    LL{i} = sol(1:pnum,i);
end

for solind = 1 : solnum % every sample point
    repvalue = repvalues(:,solind);
    newP = subs(symK,repvars,repvalue);
    ff = symp2char(newP);
    ff = repvar4inline(ff,vars);
    tmpout = verifyroots(LL,ff);
    % join the result and its sample value which replace variables together
    for i = 1 : length(tmpout)
        tmpout{i} = [tmpout{i};[repvalue,repvalue]];
    end
    if ~isempty(tmpout)
        out = [out,tmpout(:)];
    end
end

count = length(out);

end
