function x_extract = gensampleroots(num_var,t,A_eq,b,p,option)
% GENSAMPLEROOTS         Generate sample roots
%
%The instruction
%  
%   x_extract = GENSAMPLEROOTS(num_var,t,A_eq,b,p,option)
%
%  Input : 
%       num_var -- number of variables 
%       t -- order
%       A_eq -- equality matrix
%       b -- coefficient vector
%       p -- mpol type polynomials
%       option --  is a struct whose attributes include 'maxinneriter', 'tol', 'eta', 
%                         'continuation', 'schur_type' and 'fast_eig_ratio'. 
% Output :
%       x_extract -- sample roots
%
% Yijun. Zhu, 28 March 2013
%
maxinneriter = option.maxinneriter;
tol = option.tol;
eta = option.eta;
continuation = option.continuation;
schur_type = option.schur_type;
fast_eig_ratio = option.fast_eig_ratio;

n = nchoosek(num_var+t,t);
X = AFPC_BB_moment(n,A_eq,b,maxinneriter,tol,eta,continuation,schur_type,fast_eig_ratio);

d = ceil(deg(p(:))/2); % d is the max{ceil(deg(f_{i})/2)}
[M,s] = flat_extension(X,num_var,t,d,1e-3);
if ~any(M)
    disp('Solutions can not be extracted!');
    x_extract = [];
    return;
end
options=optimset('Display','iter','TolFun',1e-8);   % Option to display output
L_temp=vecnumb(num_var,t);
L=list2vec(L_temp,num_var);
if s==0
    disp('Solutions can not be extracted!');
    x_extract = [];
else
    x_extract = Extract_new(num_var,L,M,s); % Extrat solutions
   % refine the solutions by using Newton's method
    nx = size(x_extract,2);
end  

end