function [out, count] = minorVerification(symK, vars, x_extract)
% MINORVERIFICATION         Use 'minor' method to verify
%
%The instruction
%  
%   [out, count] = MINORVERIFICATION(symK, vars, x_extract)
%
%  Input : 
%       symK -- sym type polynomials 
%       vars-- variables
%       x_extract -- sample points
% Output :
%       out -- results
%       count -- number of results
%
% Yijun. Zhu, 28 March 2013
%
out = {};
for i = 1 : size(x_extract,2)
    sampleroots = x_extract(:,i);
    g = minorConstructPoly(symK,vars,sampleroots);
    ff = [char(symK),';',char(g)];
    ff = ['[',ff,']'];
    newff = repvar4inline(ff,vars);
    tmpout = verifyroots({sampleroots},newff);
    out = [out, tmpout];
end
if isempty(out)
    count = 0;
else
    count = length(out);
end
end

function g = minorConstructPoly(h,vars,s)
diff_1 = diff(h,vars(1));
diff_2 = diff(h,vars(2));
M = [diff_1,diff_2];
M_eval = subs(M,vars,s);
M_eval = M_eval/sqrt(M_eval(1)^2+M_eval(2)^2);
g = M_eval(2)*diff_1 - M_eval(1)*diff_2;
end