function newL = nullspaceCons4PosDim(symK,vars,appx)
% NULLSPACECONS4POSDIM         In the case of postive dimension, use null vector to constrcut a new polynomial system
%
%The instruction
%  
%   newL = NULLSPACECONS4POSDIM(symK,vars,appx)
%
%  Input : 
%       symK -- sym type polynomials 
%       vars -- variables
%       appx -- sample point
% Output :
%       newL -- new polynomials system

% Yijun. Zhu, 28 March 2013
%
varNum = length(vars);
system = symK;

% Generate transposition of jacobian matrix
M = jacobian(system,vars);

% Using singular value decomposition with the sample point
M_eval = subs(M,vars,appx);
[U,S,V] = svd(M_eval);

% Get indexes of null vector
SRowNum = size(S,1); % number of variables
diag = ones(1,SRowNum)*S;
index = 1 : length(diag);
index = index(diag<1e-5); 

% Pick up one null vector 
if ~isempty(index) %  pick up one null vector randomly if it exists
    randIndex = randi(length(index));
    vecIndex = index(randIndex);
    v = V(:,vecIndex);
else % pick up one random vector
%     randCoefVec = myrandom(-1,1,LNum,1);
%     v = randCoefVec;
end

% Generate new polynomial system
h = M*v;
h = h - subs(h,vars,appx);
newL = linearComb([symK(:);h], varNum);

end