function newL = nullspaceConstruct(symK,vars,appx,urv)
% NULLSPACECONSTRUCT      Use 'nullspace' method to construct (n-m)  new polynomials,  and combine  new polynomials and old polynomials into one polynomial system
%
%The instruction
%  
%   newL = NULLSPACECONSTRUCT(symK,vars,appx,urv)
%
%  Input : 
%       symK -- sym type polynomials 
%       vars -- variables
%       appx -- sample point
%       urv  -- flag to determine using random vector or not
% Output :
%       newL -- new polynomials system
%
% Yijun. Zhu, 7 April 2013
%
varNum = length(vars);
polyNum = length(symK);
if polyNum == varNum
    isSquare = true;
else
    isSquare = false;
end

u = myrandom(-1,1,1,varNum); % random vector
horiVec = vars - u; 
horiVec = horiVec.*horiVec;
g = sum(horiVec); % Generate g polynomial
system = [symK,g];

% Generate transposition of jacobian matrix
M = jacobian(system,vars);
M = M.';

% Using singular value decomposition with the sample point
M_eval = subs(M,vars,appx);
[U,S,V] = svds(M_eval,max(size(M_eval)),'L');
% [U1,S1,V1] = svd(M_eval);
% [U,S,V] = svd(M_eval)

% Get index of null vector
SRowNum = size(S,1); % number of variables
diag = ones(1,SRowNum)*S;
index = 1 : length(diag);
index = index(diag<1e-5); % Get the index of null vector

% Get vector 
if ~isempty(index) && ~urv % Exist null vector
    disp('Use null vector!')
    randIndex = randi(length(index));
    vecIndex = index(randIndex);
    v = V(:,vecIndex);
else % Not exist null vector
    disp('Use random vector!')
    randCoefVec = myrandom(-1,1,polyNum+1,1);
    v = randCoefVec;
end
    
% Generate new polynomial system
h = M*v;
% h_val = subs(h,vars,appx)
h = h - subs(h,vars,appx);
if isSquare
    newL = linearComb([symK(:);h], varNum);
else
    newL = linearComb(h,varNum - polyNum) + 1e-20;
%     newL = linearComb(h,varNum - polyNum);
    newL = [symK,newL];
end

end
