function [out, count] = nullspaceVerification(symK,vars,LL,option)
% NULLSPACEVERIFICATION         Use 'nullspace' method  to construct a new polynomials system and then verify it
%
%The instruction
%  
%   [out, count] = NULLSPACEVERIFICATION(symK,vars,LL,varargin)
%
%  Input : 
%       symK -- sym type polynomials 
%       vars -- variables
%       LL -- sample points
%       option -- is a struct whose attributes include 'maxinneriter', 'tol', 'eta', 'continuation', 'schur_type', 'fast_eig_ratio', 
%                       'refine', 'demand', 'curve', 'nonradical' and 'loop'.
%                 attribute 'loop' denotes the maximum iterations to call probabilistic algorithm 'nullspaceConstruct'.
% Output :
%       out -- results
%       count -- number of results
%
% Yijun. Zhu, 28 March 2013
%

out = {};
loop = 5;
if isfield(option,'loop') && isa(option.loop,'double')
    loop = option.loop;
end
for i = 1 : length(LL)
    for j = 1 : 2*loop   % try to construct new polynomials system several times, because of  probabilistic algorithm
        if j <= loop
            urv = false;
        else
            urv = true;
        end
        newK = nullspaceConstruct(symK,vars,LL{i},urv);
%         radius = subs(newK,vars,LL{i})
        
        % Verify
        ff = symp2char(newK);
        newff  = repvar4inline(ff,vars);
        tmpout = verifyroots(LL(i),newff);
        
        if isempty(tmpout)
            tmpout  = verifyRootsByViss(LL(i),newK,vars,1E-3);
        end
            
        % Check every sample points in 'tmpout'
        newout = {};
        count = 1;
        for k = 1 : length(tmpout)
            e = subs(symK,vars,tmpout{k}(1:length(vars),1));
            maxe = max(e);         
            if abs(maxe) < 1e-4
                newout{count} = tmpout{k}; 
                count = count + 1;
            end
        end
        
        
%         if ~isempty(tmpout)
        if ~isempty(newout)
            break;
        end
        
    end
%     out = [out,tmpout(:)];
    out = [out,newout(:)];
end

if isempty(out)
    count = 0;
else
    count = length(out);
end

end
