function [transP1,b,E,I,irowNumArr] = transform(As,Af,Ai,irowNumArr)
% TRANSFORM           Used for transforming equality matrix and inequality matrix

% Yijun. Zhu, 28 March 2013


% For example, 
%       system = [x1*2 + x2 = 0; x2^2 + x3 = 0; x3^2 + x1 = 0];
%       variables : x1, x2, x3;
%       degree : 2;
%       1, x1, x2, x3, x1^2, x1*x2, x1*x3, x2^2, x2*x3, x3^2: 10 combinations
%       we can generate 10*10 matrix(Init matrix) with these 10 combinations, and there will be 100 new variables named y1,y2,y3,....y100;
%       there are only 35 different variables.
%       so we can generate a 100*35 matrix
r = size(As,1); % number of rows
c = size(As,2); % number of columns
P1 = sparse(r,c); % matrix to store rows(yi) which represents xi, and yi is in upper triangular matrix of Init
R = sparse(r,c); % matrix to store rows(yi) except those in P1 matrix, and yi is in upper triangular matrix of Init
rtmp = sqrt(r); % number of columns and rows of init matrix
for i = 1 : c % every column
    j = find(As(:,i)==1); % find rows of nonzero element of the i column
    index = ceil(j/rtmp) > mod(j-1,rtmp); % find indices of rows(yi) in upper triangular matrix of Init
    j = j(index);
    min_j = min(j); % find the minimum row
    if length(j) > 1
        P1(min_j,i) = 1;
        j = j(2:end);
        R(j,i) = 1;
    else
        P1(min_j,i) = 1;
    end
end
% transposition of P1
transP1 = P1';

disp('Generate equality matrix')
%Process equality matrix E
if isempty(Af)
    E = [];
else
    E = Af*transP1; % Af*X=b -> Af*transP1*Y=b -> E*Y=b
    
    % set As matrix' rows(yi) that not belongs to upper triangular matrix of Init  0
    index = 1:r;
    lower_index = index(ceil(index/rtmp) <= mod(index-1,rtmp)); % row numbers of  lower triangular matrix
    [colsInd,rowsInd] = find(As.'); % find coordinate of nonzero element of As matrix
    values = ones(length(rowsInd),1);
    values(lower_index) = 0;
    upper_As = sparse(rowsInd,colsInd,values,size(As,1),size(As,2));
    
    % compute number of rows of T matrix for initialization
    TRowNum = 0;
    for i = 1 : c
        if sum(upper_As(:,i)~=0) > 1
          index = find(upper_As(:,i)~=0);
            for j = 1 : length(index) - 1
              TRowNum = TRowNum + 1;             
            end
        end
    end
    
    % generate T matrix
    rowsInd = zeros(2*TRowNum,1);
    colsInd = zeros(2*TRowNum,1);
    values = zeros(2*TRowNum,1);
    s = 0;
    for i = 1 : c
        if sum(upper_As(:,i)~=0) > 1
          index = find(upper_As(:,i)~=0);
            for j = 1 : length(index) - 1
              s = s + 1;
              rowsInd(2*s-1) = s;
              rowsInd(2*s) = s;
              colsInd(2*s-1) = index(j);
              colsInd(2*s) = index(j+1);
              values(2*s-1) = 1;
              values(2*s) = -1;              
            end
        end
    end
    T = sparse(rowsInd,colsInd,values,TRowNum,size(E,2));
  
    E = [T;E];
    % generate b , E*Y = b
    b = zeros(size(E,1),1);
    b = -E(:,1)+b;
    b = [1;b];
    E(:,1) = 0;
    fstRow = sparse(1,r);
    fstRow(1,1) = 1;
    E = [fstRow;E];
end


% Process inequality matrix I 
if isempty(Ai)
    I = [];
else
    I = [];% inequality matrix
    iNum = size(irowNumArr);
    tAi = Ai;
    for i = 1 : iNum
        irowNum = irowNumArr(i);% get number of rows of one inequality matrix
        cAi = tAi(1:irowNum,:);% get the inequality matrix
        tAi = tAi(irowNum+1:end,:);%update tAi
        ri = size(cAi,1); % number of rows of Ai matrix
        ci = size(cAi,2); % number of columns of Ai matrix
        ritmp = sqrt(ri); %
        n = (ri - ritmp)/2 + ritmp; % number of yi in upper triangular matrix
        irowNumArr(i) = n;
        tmpAi = sparse(n,ci); % matrix to store rows(yi) in upper triangular matrix
        k = 1;
        for j = 1 : ri
            if (ceil(j/ritmp) > mod(j-1,ritmp)) % check if the row is in upper triangular matrix
                tmpAi(k,:) = cAi(j,:);
                k = k+1;
            end
        end
        I = [I;tmpAi*transP1]; % Ai*X=c -> Ai*transP1*Y=c -> tmpAi*transP1*Y = c -> I*Y = c
    end
end


end