function [out,terminateFlag,count,remLL] = verifyroots(LL,ff,varargin)
% VERIFYROOTS         Use verifynlss and verifynlss2 to verify sample  points
%
%The instruction
%  
%  [out,terminateFlag,count,newLL] = VERIFYROOTS(LL,ff,varargin)
%
%  Input : 
%       LL -- sample points
%       ff -- char type polynomial system
%       varargin -- if the 3rd input is a struct whose attribute include
%                               'demand' and the value of attribute 'demand' is 
%                               'each_component', we will verify every sample point.
% Output :
%       out -- results
%       terminateFlag -- flag which tells outside loop to terminate
%       count -- number of results
%       newLL -- sample points of unsuccessful verification
%
%Tips:
%       Generally, our verified sample points are generated by verifynlss
%       or verifynlss2. However, when option.demand = each_component, our
%       verified sample points will be generated by both verifynlss and
%       verifynlss2
%
% Yijun. Zhu, 28 March 2013
%
remLL = {}; % remainder sample points

if 3 == nargin && islogical(varargin{1})
    each = varargin{1};
elseif nargin > 3
    error('too many inputs');  
else
    each = false;
end

f = inline(ff); 
count = 0;
out = {};
terminateFlag = 0;
k = 1;
for i = 1:length(LL)
    [a,b] = verifynlss(f,LL{i});
    if rad(a(1))^2 > -1 
        res = [];
        res(:,1) = inf(a);
        res(:,2) = sup(a);
%         a_radius = rad(a)
        count = count+1;
        out{count} = res(:,:);   
    else
        remLL{k} = LL{i};
        k = k + 1;
    end    
end

% update sample points
LL = remLL;
remLL = {};

if count > 0 && ~each
    terminateFlag = 1;
    disp('=========================================================')
    disp('Verifynlss generate results');
    return;
end

% Next, we try to use verifynlss2 function to find the solution interval
k = 1;
if count == 0 || each
    for i = 1:length(LL)
        [a,b] = verifynlss2(f,LL{i},1);
        if rad(a(1))^2 > -1 && abs(b(end)) <= 1e-1
            res = [];
            res(:,1) = inf(a);
            res(:,2) = sup(a);
%             a_radius = rad(a)
            disp('singular root exist')
            count = count + 1;
            out{count} = res(:,:);            
        else
            remLL{k} = LL{i};
            k = k + 1;
        end
    end
end
if count > 0
    terminateFlag = 1;
    disp('=========================================================')
    disp('verifynlss2 generate results');
    return;
end

end