function output(filename,out,vars,varargin)
% OUTPUT         Output information to file
%
%The instruction
%  
%  OUTPUT(filename,out,vars,varargin)
%
%  Input : 
%       filename -- name of file to output
%       out -- results
%       vars -- variables
%       varargin -- double type input denotes time consuming,
%                   char type input denotes relative message
%
% Yijun. Zhu, 28 March 2013
%
fid = fopen(filename,'w');
if nargin > 3 && nargin < 6
    for i = 1 : nargin - 3
        if isa(varargin{i},'double')
            t = varargin{i};
        elseif isa(varargin{i},'char')
            msg = varargin{i};
        end
    end
elseif nargin > 5
    error('Too many input');
end


% output the cost time of computation
if exist('t','var')
    fprintf(fid,'the total computational time is %f s\n\n', t);
end

% output the order of variables
varsstr = [];
for i  = 1: length(vars)
    varsstr = [varsstr,' ', char(vars(i))];
end
fprintf(fid,'\r\n');
fprintf(fid, 'the order of variables is:');
fprintf(fid,'\r\n');  
fprintf(fid,'%s',varsstr);
fprintf(fid,'\r\n');  

% ouput related information
if exist('msg','var')
    fprintf(fid,'\r\n');
    fprintf(fid, 'Information:');
    fprintf(fid,'\r\n');  
    fprintf(fid,'%s',msg);
    fprintf(fid,'\r\n');  
end

% ouput the number of singular root and regular root
singularNum = 0;
regularNum = 0;
for i = 1 : length(out)
     solDim = size(out{i},1);
     if (solDim > length(vars))
         singularNum = singularNum + 1;
     elseif (solDim == length(vars))
         regularNum = regularNum + 1;
     else
         ;
     end
end
fprintf(fid,'\r\n');
fprintf(fid, 'the number of regular roots is:');
fprintf(fid,'\r\n');  
fprintf(fid,'%d',regularNum);
fprintf(fid,'\r\n');  
fprintf(fid, 'the number of singular roots is:');
fprintf(fid,'\r\n');  
fprintf(fid,'%d',singularNum);
fprintf(fid,'\r\n');  

% output the maximum width of results
maxW = getMax(out)
fprintf(fid,'\r\n');
fprintf(fid, 'the max interval width of variables is:');
fprintf(fid,'\r\n');  
fprintf(fid,'%e',maxW);
fprintf(fid,'\r\n');  

% output every result
for i  = 1 : length(out)
    fprintf(fid,'\r\n');
    fprintf(fid, 'One real solution interval is:');
    fprintf(fid,'\r\n');  
    fprintf(fid,'%25.16f %25.16f\n',out{i}.');
    fprintf(fid,'\r\n'); 
end
fclose(fid);
end


function maxW = getMax(out)
array = [];
for i = 1 : length(out)
    array(i) = getMaxWidth(out{i});
end
maxW = max(array);
end


function maxW = getMaxWidth(sol)
maxW = 0;
for i = 1 : length(sol) % every variable interval
    tmp = sol(i,2) - sol(i,1);
    if tmp > maxW
        maxW = tmp;
    end
end
end
