function [out,terminateFlag]  = verifyRootsByViss(LL,p,vars,tol)
% VERIFYROOTSBYVISS         Use viss to verify sample  points
%
%The instruction
%  
%  [out,terminateFlag] = VERIFYROOTSBYVISS(LL,p,vars,tol)
%
%  Input : 
%       LL -- sample points
%       p -- sym type polynomial system
%       vars -- variables
%       tol -- tolerance
% Output :
%       out -- results
%       terminateFlag -- flag which tells outside loop to terminate
%
%Tips:
%      In the case of postive dimension, there will be issues in viss
%
% Yijun. Zhu, 28 March 2013
%
out = {};
terminateFlag = 0;
vissCount = 1;
for j = 1 : length(LL) % verify every sample point
    appx =  [LL{j}];
    [X,xs] = viss(p,vars,appx,tol);
    k = length(xs) - length(vars); % number of auxiliary variables
    adFlag = false;  
    % abandon the result when it is NaN
    if sum(isnan(inf(X))) > 0
        adFlag = true;
    end   
    % abandon the result when its width of intveral of auxiliary variables is too large
    if k > 0 
        for m = 1 : k % every auxiliary variables
            if abs(xs(end+1-m) ) > 0.1 
                adFlag = true;
                break;
            end % end for if abs..
        end % end for m
    end % end for if k..
    
    if adFlag == false
        res=[];
        res(:,1)=inf(X);
        res(:,2)=sup(X);
%         X_radius = rad(X)
        out{vissCount} = res(:,:);
        vissCount = vissCount + 1;
    else
        disp('Abandon the solution:');
        X
    end % end for if adFlag..
end % end for j

if ~isempty(out)
    disp('=========================================================')
    disp('Viss generate results');
    terminateFlag = 1;
end
end