function [X,xs] = viss(F,vars,appx,tol)
% VISS         Use viss to verify sample  points
%
%The instruction
%  
%  [X,xs] = VISS(F,vars,appx,tol)
%
%  Input : 
%       F -- polynomial system
%       vars -- variables
%       appx -- approximate sample point
%       tol -- tolerance
% Output :
%       X -- interval of results 
%       xs -- approximation
%
% Yijun. Zhu, 28 March 2013
%
[dflsys,dflsol] = MultipleRoots(F,vars,appx,tol);
if isempty(dflsys)
    X = infsup(NaN,NaN);
    xs = NaN;
    return;
end
sys = myrep4inline(dflsys);
f = inline(sys);
[X,xs] = verifynlss(f,dflsol);

end


%%%%%%%%%Deflation with Smoothing Parameters%%%%%%%%%%

function [dflsys,dflsol] = MultipleRoots(pol,ord,mroots,tol)

%%%%%%%%Initialization%%%%%%%%

n = length(ord);%size of input system
vars = myseq('x(i)','i',1:n);
eqs = subs(pol,ord,vars);
oprtzeros = [mroots(1:n)];
JacZero = subs(jacobian(eqs,vars),vars,mroots);
[br,leftnull,rightnull] = AppNull(JacZero,tol);
disp('MultipleRoots:');
temp = (svd(JacZero));
disp(['  Breadth and smallest singular value of Jacobian:',num2str(br),',',num2str(temp(end))]);
if br<n   
% % %     disp('MultipleRoots:');
% % %     disp(['  Singular values around tolerance:',num2str(temp(end-br)),',',num2str(temp(end+1-br))]);
else
    disp('MultipleRoots:');
    disp(['  Singular values around tolerance:',num2str(temp(end+1-br))]);
end
[rowind,colind] = CoRank(leftnull,rightnull);%rank-deficient information
numdfl = 0;%number of deflation
dfl = {eqs};%deflation system
nullvec = {};
perpos = [];
dflsz = n;%size of deflation system

count = 3; % maximum iterations
% count = 5;
%%%%%%%%Deflation%%%%%%%%

%%%%%%%%Special Case: Breadth One%%%%%%%%

if br==1
    appx = mroots(:);
    for i = 1 : 4
        appx = MultipleRootRefinerBreadthOne(eqs,vars,appx,tol);%refine approximation to high accuracy
    end
    oprtzeros = appx;
    JacZero = subs(jacobian(eqs,vars),vars,appx);
    tmpneweqs = jacobian(eqs,vars(colind(1)));
    neweqs = tmpneweqs(:);
    compl = ComplSet(colind,n);
    while br==1 
        % faliure, and return
        if (numdfl >= count)
            dflsys = [];
            dflsol = [];
            return;
        end
        
        numdfl = numdfl+1;
        tmpA = JacZero(1:end,compl(1:n-1));
        tmpb = -subs(neweqs,vars,oprtzeros);
        azeros = pinv(tmpA)*tmpb;
        oprtzeros = [oprtzeros;azeros];
        vars = [vars;myseq('x(i)','i',dflsz+1:dflsz+n-1)];
        newvec = [];
        k = 1;
        for i = 1 : n
            if i == colind(1)
                if numdfl==1 
                    newvec(i) = 1;
                    k = k+1;
                else
                    newvec(i) = 0;
                    k = k+1; 
                end
            else
                  newvec = symsplnv(newvec,i,myseq('x[i]','i',dflsz+i-k+1));
            end
        end
        newvec = newvec(:);
        nullvec = {nullvec{:},newvec};
        neweqs = neweqs+jacobian(eqs,vars(compl(1:n-1)))*newvec(compl(1:n-1));
        dflsz = dflsz+n-1;
        tmpep = expand(neweqs(1:n));
        dfl{length(dfl)+1} = tmpep(:).';
        neweqs = zeros(n,1);%criterion for next order derivative
        for i = 1 : numdfl
            neweqs = neweqs + i/(numdfl+1)*jacobian(dfl{numdfl-i+2},vars(1:n))*nullvec{i};
        end
        tempMatrix = [subs(neweqs,vars,oprtzeros),JacZero(1:end,compl(1:n-1))];
        [br,leftnull,rightnull]= AppNull(tempMatrix,tol);
        if br>0
            tmp = [subs(neweqs,vars,oprtzeros),JacZero(1:end,compl(1:n-1))];
            tmpsvd = svd(tmp);
            disp('MultipleRoots:');
            disp(['  Singular values around tolerance: ',num2str(tmpsvd(end-br)),' , ',num2str(tmpsvd(end-br+1))]); 
        end
    end
    tmpsvd = svd([subs(neweqs,vars,oprtzeros),JacZero(1:end,compl(1:n-1))]);
    disp('MultipleRoots:');
    disp(['  Number of deflation and smallest singular value of deflated system:',num2str(numdfl),' , ',num2str(tmpsvd(end))]);
    newper = 0;
    for i = 0 : numdfl-1
        newx = myseq('x[i]','i',dflsz+1+i);
        newper = newper+newx*vars(colind(1))^i;
    end
    for i = 1 : numdfl
        dfl{i}(rowind(1)) = dfl{i}(rowind(1))+newper;
        newper = 1/i*diff(newper,vars(colind(1)));
    end
    oprtzeros = [oprtzeros;zeros(numdfl,1)];
    dflsys = [dfl{:}];
    dflsol = oprtzeros;
    
%%%%%%%%General Case%%%%%%%%
    
else    
    while br > 0  
        % failure, and return
        if (numdfl >= count)
            dflsys = [];
            dflsol = [];            
            return;
        end
        
        for i = 1 : br 
            tmpx = myseq('x[j]','j',dflsz+i);
            dfl{1}(rowind(i)) = dfl{1}(rowind(i))+(1/factorial(numdfl))*tmpx*vars(colind(i))^numdfl; %add perturbations to original system	
        end
		numdfl = numdfl+1;
        vars = [vars;myseq('x[i]','i',dflsz+1:dflsz+br)];
        perpos = [perpos,dflsz+1:dflsz+br];
        oprtzeros = oprtzeros(:).'; % to promise 'oprtzeros' is horizontal vector
        oprtzeros = [oprtzeros,zeros(1,br)];
        ranvec = ones(br,1);
        newvec = zeros(dflsz,1);
		k = 1;
		for i = 1 : n
			if  k<=br && i==colind(k)
                newvec(i) = ranvec(k); 
                k = k+1; 
            else
                newvec = symsplnv(newvec,i,myseq('x[i]','i',dflsz+br+i+1-k));
            end
        end
		for i = n+1 : dflsz 
            newvec = symsplnv(newvec,i,myseq('x[i]','i',dflsz+i));
        end
        vars = [vars;myseq('x[i]','i',dflsz+br+1:2*dflsz)];
        nullvec = {nullvec{:},newvec};
		%solve a least squares problem for new parameters
		compl = ComplSet(colind,dflsz);
        tmpA = JacZero(1:end,compl(1:dflsz-br));
        tmpb = -JacZero(1:end,colind(1:br))*ranvec;
        azeros = pinv(tmpA)*tmpb;
        tmpazeros = azeros.';
        oprtzeros = [oprtzeros,tmpazeros(1:dflsz-br)];
		for i = 1 : numdfl 
            k = length(nullvec{i});
			if i < numdfl 
                tmpJacobian = jacobian(dfl{i},vars(1:k))*nullvec{i};
                tmpJacobian = tmpJacobian.';
                dfl{i+1} =[dfl{i},tmpJacobian(1:k)];
            else
                tmpJacobian = jacobian(dfl{i},vars(1:k))*nullvec{i};
                tmpJacobian = tmpJacobian.';
                dfl = {dfl{:},[dfl{i},tmpJacobian(1:k)]};
			end
		end
		dflsz = 2*dflsz;
        JacZero = subs(jacobian(dfl{end},vars),vars,oprtzeros);
		[br,leftnull,rightnull] = AppNull(JacZero,tol);
		if br>0 
			[rowind,colind] = CoRank(leftnull(end+1-n:end,1:end),rightnull(1:end,1:n));
			if br<n
                tmpsvd = svd(JacZero);
                disp('MultipleRoots:');
                disp(['  Singular values around tolerance:',num2str(tmpsvd(end-br)),' , ',num2str(tmpsvd(end))]);
            else
                tmpsvd = svd(JacZero);
                disp('MultipleRoots:');
                disp(['  Singular values around tolerance:',num2str(tmpsvd(end-br+1))]);
			end
        end
        disp('MultipleRoots:');
        disp(['  Number of deflation and breadth:',num2str(numdfl),' , ',num2str(br)]);
    end
    tmpsvd = svd(JacZero);
    disp('MultipleRoots:');
    disp(['  Number of deflation and smallest singular value of deflated system:',num2str(numdfl),' , ',num2str(tmpsvd(end))]);
    dflsys = dfl{end};
    dflsol = oprtzeros(1:dflsz);
end
    
end


%%%%%   Refine the Mutiple Root   %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%   
%	Calling sequence  MultipleRootRefinerBreadthOne(lsys,ord,appx,tol)
%      
%	Input  One polynomial system, an order, an approxiate root and a tolerance
%
%  Output  The multiplity, the index and the refined root 
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

 
% MultipleRootRefinerBreadthOne=proc(lsys,ord,approot,tol)
function  appx = MultipleRootRefinerBreadthOne(lsys,ord,approot,tol)
n = length(ord);
appx = approot;
Jac = jacobian(lsys,ord);
Jaclhs = subs(Jac,ord,appx);
disp('MultipleRootRefinerBreadthOne:');
disp('  singular values of jacobian');
disp(svd(Jaclhs));

tsvdJaclhs=svd(Jaclhs);
A=Jaclhs'*Jaclhs+tsvdJaclhs(n)*sparse(eye(n));
b=(-Jaclhs')*(subs(lsys,ord,appx).');
appy = pinv(A)*b; %alternative method of least-square method
appx = appx + appy;
disp('MultipleRootRefinerBreadthOne:');
disp('  roots after prenewton');
disp(appx);
%Digits=14
Jaclhs = subs(Jac,ord,appx);
[U,tsvdJaclhs,V]=svd(Jaclhs);
flag = tsvdJaclhs(n);
Vt = V';
temp = Vt(end,1:end)';
transtemp = temp.';
nullvector = transtemp(:);

maindir = maindirection(nullvector);
maindir = maindir(2);
tmp = maindirection(nullvector);
tmp_1 = tmp(1);
nullvector = nullvector/tmp_1;
neword = exchange(ord,ord(1),ord(maindir));
newL = exchange(Jac*nullvector,ord(1),ord(maindir));
nlsys = exchange(lsys,ord(1),ord(maindir));
Jac = jacobian(nlsys,ord);
Jac = Jac(1:end,2:end);
Jaclhs = subs(Jac,neword,appx);
if maindir == 1 
    A = nullvector(2:end).';
else
    A = nullvector([2:maindir-1,1,maindir+1:end]).';
end
L = newL;
multiplicity=2;
while flag < tol && multiplicity < 8;
    P = diff(newL,ord(1));
    Q = L*A;
    for i = 1 : n-1
        P = P + diff(Q(1:end,i),ord(i+1));
    end
    Jacrhs = subs(P,neword,appx);
    tmpflag = svd([Jacrhs,Jaclhs]);
    flag = tmpflag(n);
    disp('MultipleRootRefinerBreadthOne:');
    disp(['  The smallest singular value: ',num2str(flag)]);
    if flag > tol 
        break;
    else
        [U,tsvdJaclhs,V] = svd([Jacrhs,Jaclhs]);
        Vt = V';
        newA = (Vt(end,1:end))';
        newA = newA(:);
        newA = newA(2:end)/newA(1);
        newL = (P + Jac*newA)/multiplicity;
        A = [newA.';A];
        L = [L,newL];
        multiplicity = multiplicity + 1;
        disp('MultipleRootRefinerBreadthOne:');
        disp(['  The multiplicity',',',num2str(multiplicity)]);
    end
end
disp('MultipleRootRefinerBreadthOne:');
disp(['  The multiplicity',',',num2str(multiplicity)]);
Jacrhs = subs(P,neword,appx)/multiplicity;
A = [Jacrhs,Jaclhs];
b = -subs(newL,neword,appx);
tmplsq = pinv(A)*b;
appx = appx + tmplsq(1)/multiplicity*nullvector;

end


%%%%%%%%%%%%%%%swap variables%%%%%%%%%%%%%%%%%
function new = exchange(p,para1,para2)
p = p(:);
tmp = subs(p,para1,sym('t'));
tmp = subs(tmp,para2,para1);
new = subs(tmp,sym('t'),para2);
end


%%%%%%%%Approximate Nullspace%%%%%%%%

function [br,leftnull,rightnull] = AppNull(A,tol)
n = size(A,2);
[leftnull,SS,rightnull] = svd(A);
rightnull = rightnull';
SS = SS*ones(size(SS,2),1);
br = n;
for i = n :-1: 1
    if SS(i) > tol 
        br = n-i; 
        break;%count how many singular values are smaller than tol
    end
end
if br > 0
    leftnull = leftnull(1:end,end+1-br:end);
    rightnull = rightnull(end+1-br:end,1:end);
    return;
else
    leftnull = [];
    rightnull = [];
    return;
end
end


%%%%%%%%Complement%%%%%%%%

% ComplSet =proc(A,n)
function C = ComplSet(A,n)
C = [];
k = 1;
for i = 1 : n
    if k<=length(A) && i==A(k) 
        k = k+1;
    else
        C = [C,i];
    end
end
end


%%%%%%%%Deficiency Information%%%%%%%%

function [rowind,colind] = CoRank(A,B)
n = size(A,1);
br = size(A,2);
rowind = zeros(1,br);
colind = zeros(1,br);
leftnull = A;
rightnull = B;
for i = br :-1: 1 
    lflag = 0;
    rflag = 0;
    for j = 1 : n
        if lflag < abs(leftnull(j,i)) 
            lflag = abs(leftnull(j,i)); 
            rowind(i) = j; 
        end
        if rflag < abs(rightnull(i,j)) 
           rflag = abs(rightnull(i,j)); 
           colind(i) = j;
        end
    end
    if i > 1
        for k = 1 : i-1
            for j = 1 : n
                leftnull(j,k) = leftnull(j,k)-leftnull(j,i)*leftnull(rowind(i),k)/leftnull(rowind(i),i);
                rightnull(k,j) = rightnull(k,j)-rightnull(i,j)*rightnull(k,colind(i))/rightnull(i,colind(i));
            end
        end
    end
end
colind = sort(colind);
rowind = sort(rowind);
end


%%%%%%%%Main Direction%%%%%%%%

function maindir = maindirection(nullvector)
temp = 0;
maindir = 1;
for i = 1 : length(nullvector)
    if abs(nullvector(i))>temp
        temp = abs(nullvector(i));
        maindir = i;
    end
end
maindir = [nullvector(maindir),maindir];

end  



%%%%%%%%replace sym variables in numerical vector%%%%%%%%

function vec = symsplnv(numvec,i,symvar)
tmp = sym(numvec);
tmp(i) = symvar;
vec = tmp;
end

%%%%%%%%%%%%%%%%Replace variables with x(1),x(2),x(3)....... %%%%%%%%%%%%%%%%%%%%

function str = myrep4inline(e)
if isempty(e)
    str = '';
    return;
end
e = vpa(e);
newstrarr = myrep(char(e));
l = length(newstrarr);
for i = 1 : l
    if 1 == i
        str = [char(newstrarr(i))];
    else
        str = [str,';',char(newstrarr(i))];
    end
end
str = ['[',str,']'];
end

function newstr = myrep(oldstr)
pattern = 'x[\d]*';
oldarr = regexpi(oldstr,pattern,'match');
newarr = sprep(oldarr);
newstr = subs(oldstr,oldarr,newarr);

end

function new = sprep(old)
% old = 'x51'
new = strrep(old,'x','x(');
new = strcat(new,')');
end


%%%%%%%%%%%%%%Generate sym type variable%%%%%%%%%%%%%

function vars = myseq(template,ind,vec)
% e.g
% Input : 
% template = 'x(i)'
% ind = 'i'
% vec = 1 : n

% Output :
% vars = [x1,x2,x3,...,xn]
if (~strcmp(class(template),'char') || ~strcmp(class(ind),'char'))
    error('Error,wrong type of template or ind');
end
workTemplate = template;
old = {'(',')','[',']'};
new = {''};
vars = [];
for k = 1 : length(old)
    workTemplate = strrep(workTemplate,old{k},new{1});
end
for k = 1 : length(vec)
    varv = strrep(workTemplate,ind,num2str(vec(k)));
    vars = [vars;sym(varv)];
end
end


